﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.ComponentModel.Composition;
using MoNo;
using MoNo.Geometries;
using MoNo.Studio;
using MoNo.CSharpUtils;

namespace QuickCsTemplate
{
  [Export( typeof( MoNo.Studio.IMenuFactory ) )]
  class EntityMenuFactory : MenuFactoryForCSharp<QuickAppContext>
  {
    public override IEnumerable<MenuItem> EnumerateMenuItems( QuickAppContext con )
    {
      yield return MenuItemFactory.Folder( "Samples",
        MenuItemFactory.Folder( "Entity Samples",
          con.CreateCommand( () =>
          {
            // MoNo.Geometries.Polyline<Point3d> エンティティを生成
            var pol = new Polyline<Point3d>( false, new[]
            {
              new Point3d( 0.0, 0.0, 0.0 ),
              new Point3d( 1.0, 0.0, 0.0 ),
              new Point3d( 1.0, 1.0, 0.0 ),
              new Point3d( 0.0, 1.0, 1.0 )
            } );

            // シーングラフにエントリを登録
            con.Document.Val.Entries.Add( new MoNo.Wpf.Entry( pol ) );
          } )
          .ToMenuItem( "Polyline<Point3d>" ),

          con.CreateCommand( () =>
          {
            var tris = new Soup<Triangle<Point3d>>( new[]
            {
              new Triangle<Point3d>( new Point3d( 0.0, 0.1, 0.3 ), new Point3d( 0.2, 0.0, 0.5 ), new Point3d( 0.4, 0.3, 0.0 ) ),
              new Triangle<Point3d>( new Point3d( 0.8, 0.5, 0.4 ), new Point3d( 0.0, 0.2, 0.4 ), new Point3d( 0.2, 0.1, 0.6 ) ),
              new Triangle<Point3d>( new Point3d( 0.8, 0.6, 0.3 ), new Point3d( 0.3, 0.5, 0.4 ), new Point3d( 0.2, 0.2, 0.0 ) ),
              new Triangle<Point3d>( new Point3d( 0.5, 0.2, 0.5 ), new Point3d( 0.3, 0.6, 0.1 ), new Point3d( 0.0, 0.3, 0.7 ) )
            } );
            con.Document.Val.Entries.Add( new MoNo.Wpf.Entry( tris ) );
          })
          .ToMenuItem( "Soup<Triangle<Point3d>>" ),

          con.CreateCommand( () =>
          {
            var vertices = new[] { // 頂点配列
              new Point3d( 0.0, 0.0, 1.0 ),
              new Point3d( 1.0, 0.0, 1.0 ),
              new Point3d( 1.0, 1.0, 1.0 ),
              new Point3d( 0.0, 1.0, 1.0 )
            };
            var mesh = new Mesh<Point3d>( vertices, new[] { new Facet( 0, 1, 2 ), new Facet( 0, 2, 3 ) } );
            con.Document.Val.Entries.Add( new MoNo.Wpf.Entry( mesh ) );
          })
          .ToMenuItem( "Mesh<Point3d>" )
        ) );
    }
  }
}
