﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.ComponentModel.Composition;
using MoNo;
using MoNo.Studio;
using MoNo.CSharpUtils;
using Microsoft.FSharp.Core;

namespace QuickCsTemplate
{
  [Export( typeof( MoNo.Studio.IMenuFactory ) )]
  class OperationMenuFactory : MenuFactoryForCSharp<QuickAppContext>
  {
    public override IEnumerable<MenuItem> EnumerateMenuItems( QuickAppContext con )
    {
      yield return MenuItemFactory.Folder( "Samples",
        MenuItemFactory.Folder( "Operation Samples",

          con.CreateCommand( () =>
            ContUtil.FromEvent<MoNo.Graphics.ViewMouseEventArgs>( con.ViewContext.Events, nameof( con.ViewContext.Events.MouseClick ) )
            .ContinueWith( e => con.Messenger.Send( new MoNo.Wpf.InformationMessage( string.Format( "click pos = {0}", e.Location ), "Click Sample" ) ) ) )
          .ToMenuItem( "Click (simple)" ),

          con.CreateCommand( () =>
            con.GetLButtonClick( op => op.Cursor = System.Windows.Forms.Cursors.Hand )
            .ContinueWith( e => con.Messenger.Send( new MoNo.Wpf.InformationMessage( string.Format( "click pos = {0}", e.Location ), "Click Sample" ) ) ) )
          .ToMenuItem( "Click" ),

          con.CreateCommand( () =>
            from e1 in con.GetLButtonClick()
            from e2 in con.GetLButtonClick()
            from __ in con.Messenger.Send( new MoNo.Wpf.InformationMessage(
              string.Format( "1st click = {0}\nsnd click = {1}", e1.Location, e2.Location ), "Click Twice Sample" ) )
            select (Unit)null )
          .ToMenuItem( "Click Twice" ),

          con.CreateCommand( () =>
          {
            var p1 = System.Drawing.Point.Empty;
            var p2 = System.Drawing.Point.Empty;
            return
              from e1 in con.GetLButtonClick( op => op.Succeeded += ( sender, e ) => { p1 = p2 = op.EventArgs.Location; } )
              from e2 in con.GetLButtonClick( op =>
                {
                  op.MouseMove += ( sender, e ) => { p2 = e.Location; e1.View.Invalidate(); };
                  op.ForegroundScenes.Add( sc =>
                    {
                      using ( var scope = sc.Push() ) {
                        scope.Color = System.Drawing.Color.Turquoise;
                        sc.DrawLines( gl => gl.Vertices( p1.ToPoint2i(), p2.ToPoint2i() ) );
                      }
                    } );
                } )
              from _ in ContUtil.Create( () =>
                 {
                   // ２点 p1, p2 から Polyline3d オブジェクトを生成します。
                   var q1 = e1.View.Camera.ScreenToWorld( p1 );
                   var q2 = e1.View.Camera.ScreenToWorld( p2 );
                   var pol = new MoNo.Geometries.Polyline<Point3d>( false, new[] { q1, q2 } );
                   con.Document.Val.Entries.Add( new MoNo.Wpf.Entry( pol ) ); // エンティティを登録します。
                 } )
              select (Unit)null;
          } ).ToMenuItem( "Draw Line" ),

          con.CreateCommand( () =>
            from e in con.GetLButtonClick( op => op.Cursor = System.Windows.Forms.Cursors.Hand )
            let pick = e.View.PickAt<MoNo.Wpf.Entry>( e.Location.ToPoint2i() )
            from _ in pick != null ? con.ShowMessage( string.Format( "HitObject = {0}", pick.HitObject.Object ) ) : Cont.nop
            select (Unit)null
          ).ToMenuItem( "Object Picking" ),

          con.CreateCommand( () =>
          {
            // ボール状のハンドル（3Dビューに表示される、マウス操作可能なオブジェクト）を作成します。
            var ball = new MoNo.Ctrl.HBall( new Point3d( 0.5, 0.5, 0.5 ) );

            // ボールハンドルの MouseDown イベントで、ボールをドラッグで移動するコードを記述します。
            ball.MouseDown += ( view, e ) =>
            {
              if ( e.Button == System.Windows.Forms.MouseButtons.Left && ball.IsHighlighted ) {
                var ballpos = ball.Position;
                var mouseUp = new MoNo.Ctrl.LButtonUp( e.View );
                mouseUp.MouseMove += ( sender, e2 ) => // マウスを離すまでの間、マウスドラッグによるボールの移動を計算します
                {
                  var move = e.View.Camera.ScreenToWorld( e2.Location.ToPoint2i() - e.Location.ToPoint2i() );
                  ball.Position = ballpos + move;
                  e.View.Invalidate();
                };
                e.Interrupt( mouseUp ); // ドラッグ操作のためのイベントハンドリングを「割り込み」させます
              }
            };
            // HandleOperation に HBall ハンドルを設定してオペレーションを実行します
            return con.ByOperation( new MoNo.Ctrl.HandleOperation( con.ViewContext, ball ), _ => { } );
          } ).ToMenuItem( "HBall Handle Dragging" )
        ) );
    }
  }
}
