﻿namespace MessengerSample
open MoNo

type SampleMessage (arg : int) =
  member __.Arg = arg
  interface Wpf.IMessage<int>


type VmSampleDialog() =
  member val Text = Reactive "sample text"
  interface Wpf.IDialogModel with
    member __.DialogResult = true


type ViewModel() as this = 
  inherit Wpf.MessengerContext ()
  do
    let form = new System.Windows.Forms.Form()
    let driver = MoNo.Ctrl.OperationDriver.Default
    let op1 = MoNo.Ctrl.Operation(form)
    do
      op1.Entered.Add (fun _ -> printfn "entered")
      op1.Exited.Add (fun _ -> printfn "exited")
      op1.Attached.Add (fun _ -> printfn "attached")
      op1.Detached.Add (fun _ -> printfn "detatched")
      op1.Suspended.Add (fun _ -> printfn "suspended")
      op1.Resumed.Add (fun _ -> printfn "resumed")
    do
      printfn "1"
      driver.Run op1
    printfn "2"
    driver.Push()
    printfn "3"
    let op2 = MoNo.Ctrl.Operation(form)
    op2.Entered.Add (fun _ -> printfn "op2 entered")
    op2.Exited.Add (fun _ -> printfn "op2 exited")
    op2.Attached.Add (fun _ -> printfn "op2 attached")
    op2.Detached.Add (fun _ -> printfn "op2 detatched")
    op2.Suspended.Add (fun _ -> printfn "op2 suspended")
    op2.Resumed.Add (fun _ -> printfn "op2 resumed")
    driver.Run op2
    printfn "4"
    driver.Push()
    printfn "5"
    driver.Pop()
    printfn "6"
    driver.Abort()

    form.Show()

  member val MessageBoxCommand =
    cont {
      do! this.ShowMessage "test"
    }
    |> Wpf.Cont.toWpfCommand this None

  member val SampleMessageCommand =
    cont {
      let! result = this.Messenger.Send (SampleMessage 1234)
      do! this.ShowMessage (sprintf "result = %d" result)
    }
    |> Wpf.Cont.toWpfCommand this None

  member val ModalDialogCommand =
    cont {
      let! vm =  this.ShowModalDialog (VmSampleDialog())
      printfn "Text = %s" vm.Text.Val
    }
    |> Wpf.Cont.toWpfCommand this None

  member val ModelessDialogCommand =
    cont {
      let! vm =  this.ShowModelessDialog (VmSampleDialog())
      printfn "Text = %s" vm.Text.Val
    }
    |> Wpf.Cont.toWpfCommand this None
